from commmons import html, strip_tags
from commmons.url import get_host_url


def get_post_id(post) -> str:
    return post.attrib["data-content"].strip("post-")


def get_post_url(url, postid) -> str:
    if "page" in url:
        urlsplit = url.split("/")
        url = "".join(urlsplit[:len(urlsplit) - 2])

    return url.strip("/") + f"/post-{postid}"


def get_thumbnail_url(post, host_url):
    imgs = post.xpath(".//img")
    if not imgs:
        return None

    img_attrib = imgs[len(imgs) // 2].attrib
    for key in ("data-url", "src"):
        if key in img_attrib:
            val = img_attrib[key]
            if val.startswith("/"):
                return host_url + val
            return val

    return None


def scrape(url):
    root = html.from_url(url)
    posts = root.xpath("//article[contains(@data-content, 'post-')]")

    title_div = root.xpath("//div[contains(@class, 'p-title')]")[0]
    title = strip_tags(html_element_to_string(title_div))

    host_url = get_host_url(url)

    for p in posts:
        postid = get_post_id(p)
        yield {
            "fileid": f"fseg{postid}",
            "filename": f"{postid}-{title}",
            "sourceurl": get_post_url(url, postid),
            "thumbnailurl": get_thumbnail_url(p, host_url)
        }



from commmons import html
from commmons.url import get_host_url


def get_title(element):
    h3s = element.xpath("//h3")
    if h3s:
        return h3s[0].text.strip("\n").strip()

    return None


def get_thumbnail_path(element):
    imgs = element.xpath(".//div[@class='divimg']/img")
    if imgs:
        return imgs[0].attrib["src"]
    return None


def scrape_lgbb(url):
    host_url = get_host_url(url)
    root = html.from_url(url)

    for div in root.xpath("//div[@class='panel panel-default']"):
        title = get_title(div)
        if not title:
            continue

        thumbnail_path = get_thumbnail_path(div)
        if not thumbnail_path:
            continue

        href = div.xpath('.//a')[0].attrib["href"]
        post_id = href.split("/")[-1].split("-")[-1].split(".")[0]

        yield {
            "fileid": "lgbb" + post_id,
            "sourceurl": "vpr://" + host_url + href,
            "filename": title,
            "thumbnailurl": host_url + thumbnail_path
        }



from commmons import html
from commmons.url import get_host_url


def get_thumbnail_path(element):
    div = element.xpath(".//div[contains(@style,'background-image')]")[0]
    return div.attrib["style"].split("url('")[-1].rstrip("');")


def get_title(element):
    spans = element.xpath(".//span")
    if spans:
        return spans[0].text.strip()
    return None


def get_atag(element):
    atags = element.xpath(".//a")
    if atags:
        return atags[0]
    return None


def scrape_hnlg(url):
    host_url = get_host_url(url)
    root = html.from_url(url)

    for post in root.xpath("//a[contains(@href,'/article/')]"):
        href = post.attrib["href"]
        title = get_title(post)
        if not title:
            continue

        post_id = href.split("/")[-1].split("-")[-1].split(".")[0]

        yield {
            "fileid": "hnlg" + post_id,
            "sourceurl": "vpr://" + host_url + href,
            "filename": title,
            "thumbnailurl": host_url + get_thumbnail_path(post)
        }



import uuid

from commmons import html


def scrape_gimg(url):
    root = html.from_url(url)
    for img in root.xpath("//img[@src]"):
        yield {
            "fileid": str(uuid.uuid4()),
            "filename": str(uuid.uuid4()),
            "sourceurl": str(uuid.uuid4()),
            "thumbnailurl": img.attrib["src"]
        }



from commmons import html
from commmons.url import get_host_url


def scrape(url):
    tree = html.from_url(url)

    def get_thumbnailurl(vid):
        imgs = tree.xpath("//img[contains(@class,'thumb')]")
        for img in imgs:
            datasrc = img.attrib.get("data-src")
            if datasrc and vid in datasrc:
                return datasrc
        return None

    li_elements = tree.xpath("//li[contains(@class,'videoblock')]")
    for li in li_elements:
        vkey = li.attrib.get("data-video-vkey")
        vid = li.attrib.get("data-video-id")
        if vkey:
            href = f"view_video.php?viewkey={vkey}"
            for a in li.xpath(f"//a[@href='/{href}']"):
                title = a.attrib.get("title")
                if title:
                    file = {
                        "fileid": vkey,
                        "sourceurl": f"{get_host_url(url)}/{href}",
                        "filename": title.strip()
                    }

                    thumbnailurl = get_thumbnailurl(vid)
                    if thumbnailurl:
                        file["thumbnailurl"] = thumbnailurl

                    yield file
                    break



from commmons import html
from commmons.url import get_host_url


def _get_img(card):
    imgs = card.xpath(".//img")
    return imgs[0] if imgs else None


def scrape_v2ph(url):
    root = html.from_url(url)

    for a in root.xpath("//a[@class='media-cover']"):
        img = _get_img(a)
        if img is None:
            continue

        yield {
            "fileid": "v2ph" + a.attrib["href"].split("/")[-1].split(".")[0].split("?")[0],
            "filename": img.attrib["alt"],
            "sourceurl": "vpr://" + get_host_url(url) + a.attrib["href"],
            "thumbnailurl": img.attrib["data-src"]
        }



from commmons.hash import md5
from lxml.html import HtmlElement
from commmons import head

from hypersquirrel.util import html_from_url_with_headers


def _get_thumb_url(item: HtmlElement) -> str:
    try:
        thumbdiv = head(item.xpath(".//div[@class='thumb']"))
        return thumbdiv.attrib["style"].split("(")[-1].rstrip(");")
    except:
        pass
    return None


def scrape_cpt(url: str):
    root = html_from_url_with_headers(url)
    for item in root.xpath("//div[@class='item']"):
        atag = head(item.xpath(".//a"))
        if atag is not None:
            abs_link = atag.attrib["href"]
            if abs_link.startswith("https://cosplayporntube.com/tag"):
                continue
            yield {
                "fileid": "cpt" + md5(abs_link),
                "filename": atag.attrib["title"],
                "sourceurl": abs_link,
                "thumbnailurl": _get_thumb_url(item)
            }



from typing import Callable, Generator

from .scraper.buondua import scrape_buondua
from .scraper.cosplayporntube import scrape_cpt
from .scraper.ehen import scrape_ehen
from .scraper.fseg import scrape as scrape_fseg_html
from .scraper.googleimagesearch import scrape_gimg
from .scraper.hcos import scrape_hcos
from .scraper.hnlg import scrape_hnlg
from .scraper.lgbb import scrape_lgbb
from .scraper.nh import scrape as scrape_nh_html
from .scraper.nlgs import scrape_uulg, scrape_nlgs, scrape_lgcx
from .scraper.opendir import scrape as scrape_opendir
from .scraper.reddit import scrape_subreddit
from .scraper.sb import scrape as scrape_sb_html
from .scraper.sbgx import scrape_sbgx
from .scraper.v2ph import scrape_v2ph
from .scraper.vg import scrape_vipergirls as scrape_vg_html
from .scraper.xh import scrape as scrape_xh_html
from .scraper.xv import scrape as scrape_xv_html
from .watchlist import Watchlist


class WatchlistScraperFactory:
    def __init__(self):
        self.registry = list()

    def register_scraper(self, predicate: Callable[[Watchlist], bool],
                         scraper: Callable[[str], Generator[dict, None, None]]):
        self.registry.append((predicate, scraper,))

    def get_scraper(self, watchlist: Watchlist):
        for registry_pair in self.registry:
            condition = registry_pair[0]
            scraper = registry_pair[1]
            if condition(watchlist):
                return scraper
        # Default: opendir
        return scrape_opendir


_wsf = WatchlistScraperFactory()

get_scraper = _wsf.get_scraper
register_scraper = _wsf.register_scraper

register_scraper(lambda w: "pornhub.com" in w.url, scrape_nh_html)
register_scraper(lambda w: "vipergirls" in w.url, scrape_vg_html)
register_scraper(lambda w: "xhamster" in w.url, scrape_xh_html)
register_scraper(lambda w: "xvideos.com" in w.url, scrape_xv_html)
register_scraper(lambda w: "spankbang.com" in w.url, scrape_sb_html)
register_scraper(lambda w: "reddit" in w.url, scrape_subreddit)
register_scraper(lambda w: "v2ph" in w.url, scrape_v2ph)
register_scraper(lambda w: "hentai-cosplays" in w.url, scrape_hcos)
register_scraper(lambda w: "buondua" in w.url, scrape_buondua)
register_scraper(lambda w: "e-hentai" in w.url, scrape_ehen)
register_scraper(lambda w: "cosplayporntube" in w.url, scrape_cpt)
register_scraper(lambda w: "egirls" in w.url, scrape_fseg_html)
register_scraper(lambda w: "nlegs" in w.url, scrape_nlgs)
register_scraper(lambda w: "uuleg" in w.url, scrape_uulg)
register_scraper(lambda w: "honeyleg" in w.url, scrape_hnlg)
register_scraper(lambda w: "legbabe" in w.url, scrape_lgbb)
register_scraper(lambda w: "leg.cx" in w.url, scrape_lgcx)
register_scraper(lambda w: "superbeautygirlx" in w.url, scrape_sbgx)
register_scraper(lambda w: "porn-images-xxx" in w.url, scrape_hcos)
register_scraper(lambda w: "hentai-img" in w.url, scrape_hcos)
register_scraper(lambda w: "google" in w.url, scrape_gimg)



from . import PagedWatchlist
from .validator.paged import validate as validate_paged
from .validator.base import validate as validate_base


class WatchlistValidatorFactory:
    def __init__(self):
        self.registry = list()

    def register_validator(self, condition, validator):
        self.registry.append((condition, validator,))

    def get_validator(self, watchlist):
        for validator_pair in self.registry:
            condition = validator_pair[0]
            validator = validator_pair[1]
            if condition(watchlist):
                return validator
        raise ValueError(f"No validator found")


_wvf = WatchlistValidatorFactory()
_wvf.register_validator(lambda w: isinstance(w, PagedWatchlist), validate_paged)
_wvf.register_validator(lambda w: True, validate_base)


def get_watchlist_validator(watchlist):
    return _wvf.get_validator(watchlist)



from commmons import html
from commmons.url import get_host_url


def scrape(url):
    tree = html.from_url(url)
    host_url = get_host_url(url)

    divs = tree.xpath("//div[contains(@class, 'video-item')]")
    for div in divs:
        if "data-id" in div.attrib:
            dataid = div.attrib["data-id"]
            atags = div.xpath("./a[@class='n']")
            if atags:
                img = div.xpath("./a/picture/img")[0]
                yield {
                    "fileid": "sb" + dataid,
                    "sourceurl": host_url + atags[0].attrib["href"],
                    "filename": img.attrib["alt"],
                    "thumbnailurl": img.attrib["data-src"]
                }



import re
from typing import Generator
from uuid import uuid4

from hypersquirrel.scraperfactory import get_scraper
from hypersquirrel.watchlist import Watchlist, PagedWatchlist
from hypersquirrel.watchlist.decorator import maxitems

PAGED_PATTERN = re.compile("<\d+-\d+>")


def _create_watchlist(url: str):
    wname = uuid4()
    match = PAGED_PATTERN.findall(url)
    if len(match) == 1:
        pages = match[0].replace("<", "").replace(">", "").split("-")
        return PagedWatchlist({
            "name": wname,
            "url": url.replace(match[0], "${page}"),
            "page_min": int(pages[0]),
            "page_max": int(pages[1])
        })
    return Watchlist({
        "name": wname,
        "url": url
    })


def scrape(url: str) -> Generator[dict, None, None]:
    w = _create_watchlist(url)
    w.decorate(maxitems.decorator)
    return w.scrape(get_scraper(w))


def scrape_watchlist(w: Watchlist) -> Generator[dict, None, None]:
    scraper = get_scraper(w)
    return w.scrape(scraper)



from commmons import html


def get_title_and_url(div):
    h2s = div.xpath(".//h2[contains(@class, 'post-title')]")
    if h2s:
        a = h2s[0].xpath("./a")[0]
        return a.text, a.attrib["href"]
    return None


def get_thumbnail_url(div):
    return div.xpath(".//meta[@itemprop='image']")[0].attrib["content"]


def scrape_sbgx(url):
    root = html.from_url(url)

    for div in root.xpath("//div[@class='post-outer']"):
        title_and_url = get_title_and_url(div)
        if not title_and_url:
            continue

        fileid = "sbgx" + div.xpath('.//a[@name]')[0].attrib["name"]
        title, url = title_and_url

        yield {
            "fileid": fileid,
            "filename": title,
            "sourceurl": "vpr://" + url,
            "thumbnailurl": get_thumbnail_url(div)
        }



from commmons import html
from lxml import html

import re

_non_numeric = re.compile(r'[^\d]+')


def html_from_url_with_headers(url, referer=None):
    return html.from_url(url, headers={
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0",
        "Referer": referer or url
    })


def html_element_to_string(e):
    return html.tostring(e, encoding="utf-8").decode("utf-8")


def strip_non_numeric(s):
    return _non_numeric.sub('', s)



import feedparser
import requests
from commmons import head


def should_skip(entry):
    content = head(entry["content"])
    return content is None or "gif" not in content["value"]


def get_thumbnailurl(entry: dict):
    url_dict = head(entry.get("media_thumbnail"))
    if url_dict is not None:
        return url_dict.get("url")
    return None


def scrape_subreddit(rss_url: str):
    assert ".rss" in rss_url
    r = requests.get(rss_url, headers={"User-Agent": "debian:hypersquirrel:0.2"})
    root = feedparser.parse(r.text)

    for entry in root["entries"]:
        if should_skip(entry):
            continue

        yield {
            "fileid": entry["id"],
            "sourceurl": entry["link"],
            "filename": entry["title"],
            "thumbnailurl": get_thumbnailurl(entry)
        }



import itertools

from commmons import html, head
from commmons.hash import md5
from commmons.url import get_host_url

MAX_TAGS = 3


def _scrape_gallery(gallery, host_url):
    posts = gallery.xpath(".//div[@class='image-list-item']")
    for post in posts:
        atag = head(post.xpath(".//*[@class='image-list-item-title']/a"))
        img = head(post.xpath(".//div[@class='image-list-item-image']//img"))
        if atag is not None and img is not None:
            href = atag.attrib["href"]
            title = atag.text
            if href is not None and title is not None:
                yield {
                    "fileid": "hcos" + md5(href),
                    "filename": title.replace("\n", " "),
                    "sourceurl": "vpr://" + host_url + href,
                    "thumbnailurl": img.attrib["src"]
                }


def _scrape_search_results(root, host_url):
    galleries = root.xpath("//div[@id='display_area_image']")
    for gallery in galleries:
        yield from _scrape_gallery(gallery, host_url)


def _scrape_from_related_tags(root, host_url):
    atags = itertools.islice(root.xpath("//p[@id='detail_tag']//a"), MAX_TAGS)
    for atag in atags:
        href = atag.attrib["href"]
        new_root = html.from_url(host_url + href, headers={
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0"
        })
        yield from _scrape_search_results(new_root, host_url)


def scrape_hcos(url):
    root = html.from_url(url, headers={
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0"
    })
    host_url = get_host_url(url)

    yield from _scrape_search_results(root, host_url)
    yield from _scrape_from_related_tags(root, host_url)



import html
import json

from commmons.url import get_host_url


def get_file_divs(root):
    divs = root.xpath("//div[@data-id]")
    for div in divs:
        if "data-id" in div.attrib:
            yield div


def get_file_from_div(div, host_url):
    fileid = div.attrib["data-id"]
    atag = div.xpath(f".//a[@title and contains(@href, '{fileid}')]")[0]
    file = {
        "fileid": "xv" + fileid,
        "sourceurl": host_url + atag.attrib["href"],
        "filename": atag.attrib["title"]
    }

    imgs = div.xpath("./div/div/a/img")
    if imgs:
        file["thumbnailurl"] = imgs[0].attrib["data-src"]

    return file


def get_related_files(host_url, root):
    for script_tag in root.xpath("//script"):
        if script_tag.text:
            text = script_tag.text.strip()
            if not text.startswith("var video_related"):
                continue
            json_str = '='.join(script_tag.text.split('=')[1:]).split('];')[0] + "]"
            items = json.loads(json_str)
            for item in items:
                yield {
                    "fileid": f"xv{item['id']}",
                    "filename": str(item["tf"]),
                    "sourceurl": host_url + str(item["u"]),
                    "thumbnailurl": str(item["i"])
                }


def scrape(url):
    host_url = get_host_url(url)
    root = html.from_url(url)
    for div in get_file_divs(root):
        file = get_file_from_div(div, host_url)
        yield file

    yield from get_related_files(host_url, root)



from commmons import html
from commmons.url import get_host_url


def _get_thumbnail_path(element):
    div = element.xpath(".//div[contains(@style,'background-image')]")[0]
    return div.attrib["style"].split("url('")[-1].rstrip("');")


def _get_title(element):
    spans = element.xpath(".//span")
    if spans:
        return spans[0].text.strip()
    return None


def _s(url, prefix):
    host_url = get_host_url(url)
    root = html.from_url(url)

    for post in root.xpath("//div[contains(@class, 'col-md')]"):
        title = _get_title(post)
        if not title:
            continue

        href = post.xpath(".//a")[0].attrib["href"]
        post_id = href.split("/")[-1].split("-")[-1].split(".")[0]

        yield {
            "fileid": prefix + post_id,
            "sourceurl": "vpr://" + host_url + href,
            "filename": title,
            "thumbnailurl": host_url + _get_thumbnail_path(post)
        }


def scrape_nlgs(url):
    yield from _s(url, "nlgs")


def scrape_uulg(url):
    yield from _s(url, "uulg")


def scrape_lgcx(url):
    yield from _s(url, "lgcx")



from commmons import html, head
from commmons.hash import md5
from commmons.url import get_host_url


def _get_file(a):
    href = a.attrib["href"]
    img = head(a.xpath("./img"))
    if img is not None:
        yield {
            "fileid": "bnd" + md5(href),
            "filename": img.attrib["alt"],
            "sourceurl": "vpr://" + get_host_url(a.base) + href,
            "thumbnailurl": img.attrib["data-src"]
        }


def scrape_buondua(url: str):
    root = html.from_url(url)
    for a in root.xpath("//a[contains(@class,'item-link')]"):
        yield from _get_file(a)



from commmons.hash import md5
from htmllistparse import htmllistparse, FileEntry


def to_file(url, entry: FileEntry) -> dict:
    full_url = f"{url}/{entry.name}"
    fileid = md5(full_url)
    return {
        "fileid": fileid,
        "sourceurl": full_url,
        "filename": entry.name.strip()
    }


def scrape(url):
    clean_url = url.rstrip("/")
    cwd, listing = htmllistparse.fetch_listing(clean_url)
    return [to_file(clean_url, x) for x in listing]



from random import sample

from commmons import html
from commmons.url import get_host_url, strip_query_params
from commmons import head
from vripper.forum.threadfactory import get_thread

from hypersquirrel.util import strip_non_numeric


def sanitize_title(title, thread_id, reply_index) -> str:
    suffix = f" Reply {reply_index}" if reply_index > 0 else ""
    return title.lstrip(thread_id).replace("-", " ").strip() + suffix


def _get_reply_count(row):
    li = head(row.xpath(".//ul[contains(@class, 'threadstats')]/li"))
    if li is not None:
        atag = head(li.xpath("./a"))
        if atag is not None:
            return int(strip_non_numeric(atag.text))
    return 0


def _get_reply_indices(replies, last=3, rand=2):
    fullrange = range(0, replies + 1)
    if replies <= (last + rand):
        return fullrange

    s = sample(fullrange[:-last], rand)
    s.extend(range(replies, replies - last, -1))
    return s


def _scrape_search_results(url, tree):
    rows = tree.xpath("//div[contains(@class, 'nonsticky')]")
    for row in rows:
        title_node = head(row.xpath(".//*[@class='threadtitle']/a[contains(@class, 'title')]"))
        if title_node is not None:
            href = title_node.attrib["href"].split("?s=")[0]
            if not href.startswith('threads/'):
                continue

            thread_title = href.lstrip("threads/")
            thread_id = thread_title.split('-')[0]
            replies = _get_reply_count(row)

            for i in _get_reply_indices(replies):
                yield {
                    'fileid': thread_id + (f"r{i}" if i > 0 else ""),
                    'filename': sanitize_title(thread_title, thread_id, i),
                    'sourceurl': f"vpr://{get_host_url(url)}/{href}" + (f"?r={i}" if i > 0 else "")
                }


def _scrape_in_thread(thread_url):
    t = get_thread(thread_url)
    for reply_index, p in enumerate(t.posts):
        if len(p.images) == 0:
            continue
        f = {
            "fileid": t.id + (f"r{reply_index}" if reply_index > 0 else ""),
            "filename": t.title + (f" Reply {reply_index}" if reply_index > 0 else ""),
            "sourceurl": f"vpr://{p.url}"
        }
        img = head(p.images)
        if img:
            f.update({"thumbnailurl": img.thumb_url})
        yield f


def scrape_vipergirls(url):
    if "/threads/" in url:
        yield from _scrape_in_thread(strip_query_params(url))
    else:
        tree = html.from_url(url)
        yield from _scrape_search_results(url, tree)



from commmons import html


def get_url_and_filename(root, fileid):
    atag = root.xpath(f"//div[@data-video-id='{fileid}']/div/a")[0]
    url = atag.attrib["href"]
    return url, atag.text


def get_divs_with_fileid(root):
    divs = root.xpath("//div[contains(@class, 'video-thumb')]")
    for div in divs:
        fileid = div.attrib.get("data-video-id")
        if fileid:
            yield div


def scrape(url):
    root = html.from_url(url)

    for div in get_divs_with_fileid(root):
        fileid = div.attrib.get("data-video-id")
        sourceurl, filename = get_url_and_filename(div, fileid)
        thumbnailurl = div.xpath(".//img")[0].attrib["src"]
        yield {
            "fileid": f"xh{fileid}",
            "sourceurl": sourceurl,
            "filename": filename,
            "thumbnailurl": thumbnailurl
        }



from .base import validate as validate_base


def validate(watchlist):
    validate_base(watchlist)
    assert "${page}" in watchlist.url
    assert watchlist.page_min and isinstance(watchlist.page_min, int)
    assert watchlist.page_max and isinstance(watchlist.page_max, int)
    assert watchlist.page_min < watchlist.page_max



from commmons import head

from hypersquirrel.util import html_from_url_with_headers


def _to_post_id(post_url: str) -> str:
    return post_url.split("/g/")[-1].split("/")[0]


def _get_file(post):
    atag = head(post.xpath(".//td[contains(@class, 'glname')]/a"))
    if atag is not None:
        post_url = atag.attrib["href"]
        post_id = _to_post_id(post_url)
        if post_id.isnumeric():
            img = head(post.xpath(f"//div[@class='glthumb' and @id='it{post_id}']//img"))
            if img is not None:
                data_source = img.attrib.get("data-src")
                source = img.attrib.get("src")
                yield {
                    "fileid": "ehen" + post_id,
                    "filename": img.attrib["title"],
                    "sourceurl": "vpr://" + post_url,
                    "thumbnailurl": data_source or source
                }


def _get_posts(root):
    posts = root.xpath("//table[contains(@class, 'itg') and contains(@class, 'gltc')]//tr")
    for post in posts:
        yield from _get_file(post)


def _get_uploader_url(root) -> str:
    atag = head(root.xpath("//div[@class='gm']//div[@id='gdn']/a[contains(@href, '/uploader/')]"))
    if atag is not None:
        return atag.attrib["href"]
    return None


def scrape_ehen(url):
    root = html_from_url_with_headers(url)
    yield from _get_posts(root)

    uploader_url = _get_uploader_url(root)
    if uploader_url is not None:
        root = html_from_url_with_headers(uploader_url)
        yield from _get_posts(root)



import itertools
from types import SimpleNamespace


class Watchlist(SimpleNamespace):
    def __init__(self, config: dict):
        super(Watchlist, self).__init__(**config)

    def decorate(self, decorator):
        decorator(self)

    def validate(self, validator):
        validator(self)

    def scrape(self, scraper, url=None):
        gen = scraper(url or self.url)
        yield from itertools.islice(gen, self.max_items)


class PagedWatchlist(Watchlist):
    def get_paged_scrape_generator(self, scraper):
        for page in range(self.page_min, self.page_max + 1):
            url = self.url.replace("${page}", str(page))
            yield from super().scrape(scraper, url)

    def scrape(self, scraper):
        gen = self.get_paged_scrape_generator(scraper)
        yield from itertools.islice(gen, self.max_items)
